package trader.brokers.dukascopy.test;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.text.SimpleDateFormat;
import java.util.TimeZone;

public class Main {
	  private static long parse(String str) {
	      return parse(str, "yyyy-MM-dd HH:mm:ss.SSS", TimeZone.getTimeZone("UTC"));
	   }
	    
	   private static long parse(String dateTime, String format, TimeZone timeZone) {
	      try
	        {
	            SimpleDateFormat sdf = new SimpleDateFormat(format);
	            sdf.setTimeZone(timeZone);
	            return sdf.parse(dateTime).getTime();
	        }
	        catch(Exception ex)
	        {
	           System.exit(1);
	           return 0;
	        }
	   }
	   
	public static void main(String[] args) {

		if (args.length != 1) {
			System.out.println("args.length != 1");
			System.exit(1);
		}
		
		File currentDir = new File("test_data");
		String dir = currentDir.getAbsolutePath();
		
		if (args[0].equals("-writerOnTick")) {
			Strategy test = new Strategy(dir, true);
			test.start();
		} else if (args[0].equals("-writerReadTicks")) {
			Strategy test = new Strategy(dir, false);
			test.start();
			
			while (!test.isConnected()) {
				System.out.println("not connected");
				try {
					Thread.sleep(1000);
				} catch (Exception ex) {
					
				}
			}
			
			System.out.println("connected");
			
			File fdir = new File(dir);
			File[] files = fdir.listFiles();
			try {
				for (File file : files) {
					String fileName = dir + "/" + file.getName();
					BufferedReader reader = new BufferedReader(new FileReader(fileName));
		            String line;
		             
		            long start = 0;
		            long end = 0;
		            
		            while ((line = reader.readLine()) != null) {
		               String[] sArray = line.split(",");
		               long time = parse(sArray[0]);
		               
		               if (start == 0) {
		            	   start = time;
		               }
		               
		               end = time;
		            }
		            reader.close();
		             
					Listener listener = new Listener(fileName + ".readTicks.csv");
					test.getTicks(start, end, listener);
					
					while (!listener.isFinished()) {
						try {
							Thread.sleep(1000);
						} catch (Exception ex) {
							
						}
					}
					
				}
				System.exit(0);
			} catch (Exception ex) {
				ex.printStackTrace();
				System.exit(1);
			}
			
		} else {
			System.out.println("bad args");
			System.exit(1);
		}
	}
}
