package trader.brokers.dukascopy.test;

import java.io.File;
import java.io.FileWriter;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.TimeZone;

import com.dukascopy.api.IAccount;
import com.dukascopy.api.IBar;
import com.dukascopy.api.IContext;
import com.dukascopy.api.IMessage;
import com.dukascopy.api.IMessage.Type;
import com.dukascopy.api.IHistory;
import com.dukascopy.api.IStrategy;
import com.dukascopy.api.ITick;
import com.dukascopy.api.Instrument;
import com.dukascopy.api.JFException;
import com.dukascopy.api.Period;
import com.dukascopy.api.system.ClientFactory;
import com.dukascopy.api.system.IClient;

public class Strategy extends Thread implements IStrategy {

	private String dir;
	private FileWriter fileWriter;
	private IClient client;
	private volatile boolean connected = false;
	private boolean write = true;
	private IHistory history;
	
	public Strategy(String dir, boolean write) {
		super();
		this.dir = dir;
		this.write = write;
	}

	private String format(long time) {
		try {
			Date date = new Date(time);

			SimpleDateFormat sdf = new SimpleDateFormat(
					"yyyy-MM-dd HH:mm:ss.SSS");
			sdf.setTimeZone(TimeZone.getTimeZone("UTC"));
			return sdf.format(date);

		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(1);
		}

		return null;
	}

	@Override
	public void onStart(IContext context) throws JFException {
		history = context.getHistory();
	}

	@Override
	public void onTick(Instrument instrument, ITick tick) throws JFException {
		if (!connected) {
			System.out.println("connected");
			connected = true;
		}
		
		if (!write) {
			return;
		}
		
		try {
			if (fileWriter == null) {
				File d = new File(dir);
				if (!d.exists()) {
					d.mkdirs();
				}
				fileWriter = new FileWriter(dir + "/newConnection-" + instrument.toString().replace("/", "") + "-" + format(System.currentTimeMillis()) + ".csv");
			}
			
			String str = format(tick.getTime()) + "," + tick.getBid() + "," + tick.getAsk() + "\n";
			System.out.print(instrument.toString().replace("/", "") + "," + str);
			fileWriter.write(str);
			fileWriter.flush();
			
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(1);
		}
	}

	public boolean isConnected() {
		return connected;
	}

	public void getTicks(long start, long end, Listener listener) {
		try {
			history.readTicks(Instrument.EURUSD, start, end, listener, listener);
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(1);
		}
	}
	
	@Override
	public void onBar(Instrument instrument, Period period, IBar askBar,
			IBar bidBar) throws JFException {
		// TODO Auto-generated method stub

	}

	@Override
	public void onMessage(IMessage message) throws JFException {
		Type type = message.getType();
		switch (type) {
		case CONNECTION_STATUS:
			if (!client.isConnected()) {
				System.out.println("not connected");
				if (fileWriter != null) {
					try {
						fileWriter.close();
					} catch (Exception ex) {
						
					}
				}
				fileWriter = null;
				connected = false;
			}
			break;
		}

	}

	@Override
	public void onAccount(IAccount account) throws JFException {
		// TODO Auto-generated method stub

	}

	@Override
	public void onStop() throws JFException {
		// TODO Auto-generated method stub

	}

	@Override
	public void run() {
		String jnlpHttpUrl_Real = "https://www.dukascopy.com/client/live/jclient/jforex.jnlp";
		String userName_Real = System.getProperty("jforex.username");
		String password_Real = System.getProperty("jforex.password");
		
		while (true) {
			while (true) {
				if (!connected) {
					break;
				}
				
				try {
					Thread.sleep(1000);
				} catch (Exception ex) {
					
				}
			}
			
			while (true) {
				try {
					client = ClientFactory.getDefaultInstance();
					
					client.connect(jnlpHttpUrl_Real, userName_Real, password_Real);
					
					int i = 10; // wait max ten seconds
					while (i > 0 && !client.isConnected()) {
						Thread.sleep(1000);
						i--;
					}
		
					if (client.isConnected()) {
						Map<Long, IStrategy> sy = client.getStartedStrategies();
						
						if (sy == null || !sy.values().contains(this)) {
							Set<Instrument> instruments = new HashSet<Instrument>();
							instruments.add(Instrument.EURUSD); 
				
							client.setSubscribedInstruments(instruments);
							client.startStrategy(this);
						}
						
						try {
							Thread.sleep(10000);
						} catch (Exception ex) {
							
						}
						
						break;
					}
					
					
				} catch (Exception ex) {
					System.out.println(format(System.currentTimeMillis()) + " connect: " + ex.getClass().getSimpleName());
					
					try {
						Thread.sleep(1000);
					} catch (Exception ex2) {
						
					}
					
				}
			}
		}
	}
}
